/*	**************************************************************************************
    *									imdbp_mymovies.js
    *									=================
    *
    *	Author: Geva Zeichner
    *	Forum:  http://forums.mozillazine.org/viewtopic.php?t=408043
    *	Date:   20/04/2008
    *	Edited: 14/07/2008
    **************************************************************************************/

window.addEventListener("load", function(e){MyMovies.Handle_Load(e);}, false);

var MyMovies =
{
	TITLE_REGEXP: /^((http:\/\/)(\w*\.)?((imdb\.com)(.*)(\/title\/tt\d{7})(\/)?))/,
	
	/***************************************************************************************
	 *	Handle_Load(e)
	 *	==============
	 *	Starting point of the module.
	 *	Registers the listeners.
	 ***************************************************************************************/
	Handle_Load: function(e)
	{
		window.document.addEventListener("DOMContentLoaded",
			function(e){MyMovies.Handle_DOM_Content_Loaded(e);}, true);
	},
	
	/***************************************************************************************
	 *	Handle_DOM_Content_Loaded(e)
	 *	============================
	 *	Recognizes when an IMDb page was loaded and then initiates the integration.
	 ***************************************************************************************/
	Handle_DOM_Content_Loaded: function(e)
	{
		// If an HTML page was loaded
		if ((e.originalTarget instanceof HTMLDocument) &&
		    (imdb_preview.prefs.getBoolPref('enable_mymovies')))
		{
			var doc = e.originalTarget.firstChild.ownerDocument;
			var url = doc.location.href;
			
			// If we loaded a movie page
			if (url.match(this.TITLE_REGEXP))
			{
				// Get the movie's id number
				this.movie_id = url.substring(url.indexOf("/tt") + 3, url.indexOf("/tt") + 3 + 7);
				
				// Integrate the "my movies" section to the movie page
				this.Inegrate_MyMovies(doc);
			}
		}
	},
	
	/***************************************************************************************
	 *	Inegrate_MyMovies(loaded_doc)
	 *	=============================
	 *	Integrates the "my movies" section to the movie page
	 ***************************************************************************************/
	Inegrate_MyMovies: function(loaded_doc)
	{
		// Create the "my movies" section
		this.my_list = loaded_doc.createElement("div");
		this.my_list_header = loaded_doc.createElement("div");
		this.my_list.appendChild(this.my_list_header);
		this.my_list_add = loaded_doc.createElement("div");
		this.my_list.appendChild(this.my_list_add);
		this.my_list_status = loaded_doc.createElement("div");
		this.my_list.appendChild(this.my_list_status);
		this.my_list_categories = loaded_doc.createElement("div");
		this.my_list.appendChild(this.my_list_categories);
		
		// Place the new section
		var side_bar = loaded_doc.getElementById("tn15lhs");
		var quick_links = side_bar.getElementsByTagName("h6")[0];
		side_bar.insertBefore(this.my_list, quick_links);
		
		// Write to the list
		this.my_list_header.innerHTML = "<h6 style=\"margin-top: 4px;\">My Movies</h6>";
		quick_links.setAttribute("style", "");
		
		// Create a request object
		this.http_req = new XMLHttpRequest();
		
		var login_area = loaded_doc.getElementById("nb15personal").
		                            getElementsByTagName("a")[0];
		
		// If we're logged out
		if (login_area.getAttribute("href") == "/register/login")
			this.my_list_status.innerHTML = "Login to load.";
		else
			this.Load_MyMovies();
	},
	
	/***************************************************************************************
	 *	Load_MyMovies()
	 *	===============
	 ***************************************************************************************/
	Load_MyMovies: function()
	{
		this.my_list_status.innerHTML = "Loading";
		
		// Load the main mymovies list
		this.http_req.open('GET', "http://www.imdb.com/mymovies/list", true);
		
		// Set handler for when finished loading
		this.http_req.onreadystatechange = function(){MyMovies.Handle_Request_Ready();};
		
		// Start the loading
		this.http_req.send(null);
	},
	
	/***************************************************************************************
	 *	Handle_Request_Ready()
	 *	======================
	 ***************************************************************************************/
	Handle_Request_Ready: function()
	{
		// If loading completed
		if (this.http_req.readyState == 4)
		{
			// Clear the status
			this.my_list_status.innerHTML = "";
			
			// If an error occurd
			if (this.http_req.status != 200)
			{
				// Set status with error
				this.my_list_status.innerHTML = "Error loading :-(";
			}
			
			// If no error occurd
			else
			{
				// Get the mymovies list's source code
				var list_source = this.http_req.responseText;
				
				// Create a list object
				var list = new MyMovies_List(list_source);
				
				// Get the categories our movie is in
				var movie_categories = list.Get_Movie_Categories(this.movie_id);
				
				var categories_source = "";
				
				// If movie is in one of the categories
				if (movie_categories.length > 0)
				{
					// Iterate over the categories
					for each (var category in movie_categories)
					{
						// Put out the category our movie is in
						categories_source += "<a class=\"link\">" +
						                     category.name +
						                     "</a>";
					}
				}
				
				this.my_list_categories.innerHTML = categories_source;
				
				// Add to category
				var add_source = "<div style='width: 77%; float:left; padding-right: 2px;'>" +
				                 "<select id='mymovies_select' style='width: 100%;'>" +
				                 "<option value='pending'>Pending</option>";
				
				// If we have categories
				if (list.categories.length > 0)
				{
					// Iterate over the categories
					for each (var category in list.categories)
					{
						// Add category to our options
						add_source += "<option value='l=" + category.id + "'>" +
						              category.name +
						              "</option>";
					}
				}
				
				add_source += "</select>" +
				              "</div>" +
				              "<img id='mymovies_submit' src='chrome://imdb_preview/skin/star.png'>";
				this.my_list_add.innerHTML = add_source;
				
				// Listen to the click
				this.my_list_add.ownerDocument.getElementById("mymovies_submit").
					addEventListener("click", function(e){MyMovies.Add_Movie(e);}, false);
			}
		}
	},
	
	/***************************************************************************************
	 *	Add_Movie(e)
	 *	============
	 ***************************************************************************************/
	Add_Movie: function(e)
	{
		var list_id = this.my_list_add.ownerDocument.getElementById("mymovies_select").value;
		
		// Add movie to list
		this.http_req.open('GET', "http://www.imdb.com/mymovies/list?" + list_id +
		                          "&add=" + this.movie_id, true);
		
		// Set status
		this.my_list_status.innerHTML = "Adding";
		
		// Set handler for when finished loading
		this.http_req.onreadystatechange = function(){MyMovies.Handle_Submit_Load();};
		
		// Start the loading
		this.http_req.send(null);
	},
	
	/***************************************************************************************
	 *	Handle_Submit_Load()
	 *	====================
	 ***************************************************************************************/
	Handle_Submit_Load: function()
	{
		// If loading completed
		if (this.http_req.readyState == 4)
		{
			// If an error occurd
			if (this.http_req.status != 200)
			{
				// Set status with error
				this.my_list_status.innerHTML = "Error loading :-(";
			}
			
			// If no error occurd
			else
			{
				this.Load_MyMovies();
			}
		}
	}
};

function MyMovies_List(source_code)
{
	this.movies = new Array();
	this.categories = new Array();
	this.Populate_Movies(source_code);
	this.Populate_Categories(source_code);
}

MyMovies_List.prototype =
{
	movies: new Array(),
	categories: new Array(),
	
	/***************************************************************************************
	 *	Populate_Movies(source_code)
	 *	============================
	 ***************************************************************************************/
	Populate_Movies: function(source_code)
	{
		// Find the start of our list
		var list_pos = source_code.indexOf("<form name=\"list\"");
		if (list_pos != -1)
		{
			var first_tr_pos = source_code.indexOf("<tr>", list_pos);
			var last_movie_pos = source_code.lastIndexOf("<a href=\"/title/tt");
			var last_tr_pos = source_code.indexOf("</tr>", last_movie_pos) + 5;
			var list_source = source_code.substring(first_tr_pos, last_tr_pos);
			
			// Get the first movie row pos
			var curr_tr_pos = list_source.indexOf("<tr>");
			
			// Go over all the rows
			while (curr_tr_pos != -1)
			{
				// Get curr movie row source
				var curr_tr_end_pos = list_source.indexOf("</tr>", curr_tr_pos) + 5;
				var curr_row_source = list_source.substring(curr_tr_pos, curr_tr_end_pos);
				
				// Get the curr movie's id
				var title_id = imdb_preview.Retrieve_String(curr_row_source,
				                             "<a href=\"/title/tt",
				                             "<a href=\"/title/tt",
				                             "/\">");
				
				// Get the curr movie's name
				var title_name = imdb_preview.Retrieve_String(curr_row_source,
				                             "<a href=\"/title/tt",
				                             "/\">",
				                             "</a>");
				
				// Get the curr category's id
				var category_id = imdb_preview.Retrieve_String(curr_row_source,
				                             "<a href=\"list?l=",
				                             "<a href=\"list?l=",
				                             "\">");
				
				// Get the curr category's name
				var category_name = imdb_preview.Retrieve_String(curr_row_source,
				                             "<a href=\"list?l=",
				                             "\">",
				                             "</a>");
				
				// Build a movie structure and add it to our movie array
				this.movies.push({id: title_id,
				                  name: title_name,
				                  category: {id: category_id,
				                             name: category_name}});
				
				// Get the next movie row pos
				curr_tr_pos = list_source.indexOf("<tr>", curr_tr_end_pos);
			}
		}
	},
	
	/***************************************************************************************
	 *	Populate_Categories(source_code)
	 *	================================
	 ***************************************************************************************/
	Populate_Categories: function(source_code)
	{
		// Get the source that is relevent to the categories
		var categories_start_pos = source_code.indexOf("<A HREF=\"list\">");
		var categories_end_pos = source_code.indexOf("</table>", categories_start_pos);
		var categories_source = source_code.substring(categories_start_pos,
		                                              categories_end_pos);
		
		// Get first category pos
		var curr_cat_pos = categories_source.indexOf("<a href=\"list?l=");
		
		// Go over all the categories
		while (curr_cat_pos != -1)
		{
			// Get curr category source
			var curr_cat_end_pos = categories_source.indexOf("</span>", curr_cat_pos) + 7;
			var curr_cat_source = categories_source.substring(curr_cat_pos, curr_cat_end_pos);
			
			// Get the curr category's id
			var cat_id = imdb_preview.Retrieve_String(curr_cat_source,
				                             "<a href=\"list?l=",
				                             "<a href=\"list?l=",
				                             "\">");
			
			// Get the curr category's name
			var cat_name = imdb_preview.Retrieve_String(curr_cat_source,
				                             "<span class=\"list\">",
				                             "<span class=\"list\">",
				                             "</span>");
			
			
			// Build a category structure and add it to our categories array
			this.categories.push({id: cat_id,
			                      name: cat_name});
			
			// Get next category pos
			curr_cat_pos = categories_source.indexOf("<a href=\"list?l=", curr_cat_end_pos);
		}
	},
	
	/***************************************************************************************
	 *	Get_Movie_Categories(movie_id)
	 *	==============================
	 ***************************************************************************************/
	Get_Movie_Categories: function(movie_id)
	{
		var categories = new Array();
		for each (var curr_movie in this.movies)
		{
			if (curr_movie.id == movie_id)
			{
				var category_exists = false;
				
				for each (var curr_category in categories)
				{
					if (curr_category.id == curr_movie.category.id)
						category_exists = true;
				}
				
				if (!category_exists)
					categories.push(curr_movie.category);
			}
		}
		
		return categories;
	}
};